@@ -1,6 +1,6 @@
 <?php {
   /*
-   * $Id: dino_form.php,v 1.37 2003/03/12 15:12:01 tumf Exp $
+   * $Id: dino_form.php,v 1.41 2003/04/01 11:06:32 nishina Exp $
    * Copyright (c) 2002-2003 Dino Co.,Ltd.
    */
 
@@ -44,6 +44,9 @@
     var $select_array_hour = array();
     var $select_array_min = array();
 
+    //᥽åɤλ   ǥեȤPOST
+    var $use_post_method = true;
+
     // ---------- private ----------
     function init_select_array_date() {
       $cur_year  = date("Y");
@@ -64,6 +67,14 @@
       }
     }
 
+    function specify_method() {
+      if($this->use_post_method){
+	return "POST";
+      }else {
+	return "GET";
+      }
+    }
+
     function set_action($action) {
       $this->action = $action;
     }
@@ -180,7 +191,7 @@
 	      if (ereg("^(sys[-:])?start$", $regs[1])) {
 		$form_started = true;
 		$tvars[$v] = $this->form_data->get_start($jvs_name,
-							 "",
+							 $this->specify_method(),
 							 $this->action,
 							 "",
 							 $this->classname
@@ -224,7 +235,7 @@
       //$i = $form_prefix . "_start";
       //$tvars[$i] = $this->form_data->get_start($jvs_name, "", $this->action);
       return $this->form_data->get_start($jvs_name,
-					       "",
+					       $this->specify_method(),
 					       $this->action,
 					       "",
 					       $this->classname
