@@ -1,6 +1,6 @@
 <?php {
   /*
-   * $Id: dino_sql.php,v 1.68 2003/03/11 02:23:58 hanawa Exp $
+   * $Id: dino_sql.php,v 1.75 2003/04/14 07:05:29 nishina Exp $
    * Copyright(c) 2001-2003 Dino Co.,Ltd. All rights reserved.
    */
 
@@ -29,13 +29,12 @@
    */
   class Dino_SQL {
     var $main_table = "";
-    var $table = array(); // ơ֥̾롣xxx as A ߤʤΤ
+    var $table = array(); // obsolete
 
-    var $tableobj = array(); // ơ֥륪֥Ȥ롣
+    var $tableobj = array(); // joinDino_SQL classΥ󥹥󥹤Ǽ
     var $alias = "";
     var $is_usedalias = array();
 
-
     var $select = array(); // 󤬶ʤ*ʳʤ,ڤŸ
     var $where = array();  // 
 
@@ -176,32 +175,55 @@
      */
     function get_all_where_clause($alias = "") {
       $where_array = array();
-      foreach($this->where as $where) {
-	$where_array[] = $where;
+      if ($alias) {
+	// join˥ơ֥̾դ
+	foreach($this->where as $where) {
+	  $where_array[] = $alias.".".$where;
+	}
+	return implode(" and ", $where_array);
       }
-      return sprintf(" where %s", implode(" and ", $where_array));
+
+      return implode(" and ", $this->where);
     }
 
-
-    function traverse_sql_tree(&$table_array, $is_used = array()) {
+    function traverse_sql_tree(&$select_array, &$from_array,
+			       &$where_array, &$orderby_array,
+			       $is_used = array()) {
       $alias = $this->get_alias();
       if ($is_used[$alias]) {
 	// error!
 	$array = array();
 	return false;
       }
-      $is_used[$alias] = true;
-      $table_array[$alias] = $this->get_from_clause($alias);
 
-      // $this->tableobj traverse
-      foreach($this->tableobj as $obj) {
-	$ret = $obj->traverse_sql_tree(&$table_array, &$is_used);
-	if ($ret == false) {
-	  $table_array = array();
-	  return false;
+      $is_used[$alias] = true;
+      $from_array[$alias] = $this->get_from_clause($alias);
+      foreach($this->where as $where) {
+	if (preg_match("/^[A-Za-z_]+[^A-Za-z_.]/", $where)) {
+	  $where_array[] = $alias.".".$where;
+	} else {
+	  $where_array[] = $where;
 	}
       }
 
+      // ŪorderbyˤΤǡϤ⤦ץˤʤϤ
+      if (!empty($this->orderby)) {
+	$orderby = $this->orderby;
+	if ($this->desc) {
+	  $orderby .= sprintf(" desc");
+	}
+	$orderby_array[] = $alias.".".$orderby;
+      }
+      // $this->tableobj traverse
+      foreach($this->tableobj as $obj) {
+	$ret = $obj->traverse_sql_tree(&$select_array, &$from_array,
+				       &$where_array, &$orderby_array,
+				       &$is_used);
+	if ($ret == false) {
+	  $from_array = array();
+	  return false;
+	}
+      }
       return true;
     }
 
@@ -543,8 +565,10 @@
 	$op = "=";
       }
 
-      $this->where["${field}${opid}"] = 
-	 $this->get_where_clause_op($field, $op, $value);
+      $where = $this->get_where_clause_op($field, $op, $value);
+      if ($where) {
+	$this->where["${field}${opid}"] = $where;
+      }
     }
 
     /**
@@ -583,7 +607,7 @@
       if ($value == "") {
 	return "";
       } else {
-	return sprintf("%s %s %s", $field, $op, $value);
+	return sprintf("%s%s%s", $field, $op, $value);
       }
     }
 
@@ -837,11 +861,16 @@
      *@param object Dino_SQL $obj 礷Dino_SQL֥ 
      *@param string $fields joinǷ礹ե
      */
-    function set_condition_join_using($obj, $fields){
+    function set_condition_join_using($obj, $fields, $cfields=""){
+
+      if(!$fields) {
+	$cfields = $fields;
+      }
       $field = $fields;
+      $cfield = $cfields;
       $this->where["${field}="] = sprintf("%s.%s=%s.%s", 
 					  $this->get_alias(), $field,
-					  $obj->get_alias(), $field);
+					  $obj->get_alias(), $cfield);
     }
     /**
      * ơ֥η(join on)
@@ -851,10 +880,11 @@
      *@param string $fields joinǷ礹ե
      *@return boolean
      */
-    function join_using(&$obj, $fields) {
+    function join_using(&$obj, $fields, $fields2="") {
       $this->join($obj);
-      $this->set_condition_join_using($obj, $fields);
+      $this->set_condition_join_using($obj, $fields, $fields2);
     }
+
     // ¸ߤʤä""֤
     function get_joined_obj($alias){
       return $this;
@@ -960,23 +990,49 @@
       } else {
 	$this->table = array($this->main_table);
       }
+
       if (sizeof($this->tableobj) <= 0) {
 	// for backward-compatibility
-	$tables = implode(",", $this->table);
-      } else {
-	$table_array = array();
-	$ret = $this->traverse_sql_tree(&$table_array);
-	if ($ret == true) {
-	  $tables = implode(",", $table_array);
+	$select_array = array($select);
+	if (sizeof($this->table) >= 1) {
+	  $from_array = $this->table;
 	} else {
-	  // ???
-	  $tables = $this->main_table;
+	  $from_array = array($this->main_table);
+	}
+	$where_array = $this->where;
+
+	// ŪorderbyˤΤǡϤ⤦ץˤʤϤ
+	if (!empty($this->orderby)) {
+	  $orderby = $this->orderby;
+	  if ($this->desc) {
+	    $orderby .= sprintf(" desc");
+	  }
+	  $orderby_array = array($orderby);
+	}
+
+      } else {
+	$select_array = array();
+	$from_array = array();
+	$where_array = array();
+	$orderby_array = array();
+
+	$ret = $this->traverse_sql_tree(&$select_array, &$from_array,
+					&$where_array, &$orderby_array);
+
+	$select_array = array($select);
+
+	if ($ret == false) {
+	  // error occured
+	  return "-- traverse_sql_tree() failed.";
 	}
       }
-      $sql = sprintf("select %s from %s", $select, $tables);
 
-      if (sizeof($this->where)) {
-	$sql .= sprintf(" where %s", implode(" and ", $this->where));
+      $sql = sprintf("select %s from %s", 
+		     implode(",", $select_array),
+		     implode(",", $from_array));
+
+      if (sizeof($where_array)) {
+	$sql .= sprintf(" where %s", implode(" and ", $where_array));
       }
       if (!empty($this->groupby)) {
 	$sql .= sprintf(" group by %s", $this->groupby);
@@ -984,11 +1040,8 @@
       if (!empty($this->having)) {
 	$sql .= sprintf(" having %s", $this->having);
       }
-      if (!empty($this->orderby)) {
-	$sql .= sprintf(" order by %s", $this->orderby);
-	if ($this->desc) {
-	  $sql .= sprintf(" desc");
-	}
+      if (sizeof($orderby_array)) {
+	$sql .= sprintf(" order by %s", implode(",", $orderby_array));
       }
       if ($this->limit > 0)  { $sql .= sprintf(" limit %d", $this->limit); }
       if ($this->offset > 0) { $sql .= sprintf(" offset %d", $this->offset); }
@@ -1103,14 +1156,9 @@
       }
 
       if (empty($insert_f) || empty($insert_v)) {
-	if (sizeof($this->insert_fields) == 0) {
-	  // ¾ˤɤȡ
-	  $sql = sprintf("insert into %s default values", $this->main_table);
-	} else {
-	  // ƤˤĤƥǥեͤ줿ǽ⤢뤬
-	  // errorȤߤʤ٤
-	  $sql = "";
-	}
+	// ƤˤĤƥǥեͤ줿Ȥߤʤ
+	// SQLoracleǤ̤ʤ
+	$sql = sprintf("insert into %s default values", $this->main_table);
       } else {
 	$sql = sprintf("insert into %s(%s) values(%s)",
 		       $this->main_table,
@@ -1199,6 +1247,516 @@
       return $sql;
     }
 
+    /**
+     * Ϳ줿SQLɤɤ֤
+     *
+     * Ϳ줿SQLɤɤ֤ޤ
+     * ơ֥̾ե̾SQLɤȽŤʤäƤʤ
+     * åΤ˻Ȥޤ
+     *
+     * PostgreSQL,MySQL,Oracle,SQL99,SQL92,
+     * 줫ΥɤǤХɤȤƤޤ
+     *
+     * ɬ׽ʬǤʤ̵̣ʤΤǡõƤޤ
+     *
+     * http://www.postgresql.jp/document/pg721doc/user/sql-keywords-appendix.htmlʷ̾ơ֥̾ˤϤǤʤޤޤƤʤ
+     * http://www.simosimo.info/know/oracle/keyword.htm
+     * http://www.mysql.com/doc/en/Reserved_words.htmlʷ̾ơ֥̾ˤϤǤʤޤޤƤʤ
+     * http://www.postgresql.org/docs/view.php?version=7.3&idoc=0&file=sql-keywords-appendix.htmlʷ̾ơ֥̾ˤϤǤʤޤޤƤʤ
+     * 
+     * http://www.mysql.gr.jp/jpdoc/3.23.x/manual.ja_Reference.html
+     * ʰŪñ줬¿̤褦˸ñ줬¿äŬѤ
+     *
+     *@access public
+     *@param string $identifier ̻ҤȤƻȤĤñ
+     */
+
+    function is_sql_keyword ($identifier) {
+      $is_keyword = array(
+			  "ABSOLUTE"            => 1,
+			  "ACCESS"              => 1,
+			  "ACTION"              => 1,
+			  "ADD"                 => 1,
+			  "ADMIN"               => 1,
+			  "AFTER"               => 1,
+			  "AGGREGATE"           => 1,
+			  "ALIAS"               => 1,
+			  "ALL"                 => 1,
+			  "ALLOCATE"            => 1,
+			  "ALTER"               => 1,
+			  "ANALYSE"             => 1,
+			  "ANALYZE"             => 1,
+			  "AND"                 => 1,
+			  "ANY"                 => 1,
+			  "ARE"                 => 1,
+			  "ARRAY"               => 1,
+			  "AS"                  => 1,
+			  "ASC"                 => 1,
+			  "ASSERTION"           => 1,
+			  "AT"                  => 1,
+			  "AUDIT"               => 1,
+			  "AUTHORIZATION"       => 1,
+			  "AUTO_INCREMENT"      => 1,
+			  "AVG"                 => 1,
+			  "BDB"                 => 1,
+			  "BEFORE"              => 1,
+			  "BEGIN"               => 1,
+			  "BERKELEYDB"          => 1,
+			  "BETWEEN"             => 1,
+			  "BIGINT"              => 1,
+			  "BINARY"              => 1,
+			  "BIT"                 => 1,
+			  "BIT_LENGTH"          => 1,
+			  "BLOB"                => 1,
+			  "BOOLEAN"             => 1,
+			  "BOTH"                => 1,
+			  "BREADTH"             => 1,
+			  "BTREE"               => 1,
+			  "BY"                  => 1,
+			  "CALL"                => 1,
+			  "CASCADE"             => 1,
+			  "CASCADED"            => 1,
+			  "CASE"                => 1,
+			  "CAST"                => 1,
+			  "CATALOG"             => 1,
+			  "CHANGE"              => 1,
+			  "CHAR"                => 1,
+			  "CHARACTER"           => 1,
+			  "CHARACTER_LENGTH"    => 1,
+			  "CHAR_LENGTH"         => 1,
+			  "CHECK"               => 1,
+			  "CID"                 => 1,
+			  "CLASS"               => 1,
+			  "CLOB"                => 1,
+			  "CLOSE"               => 1,
+			  "CLUSTER"             => 1,
+			  "COALESCE"            => 1,
+			  "COLLATE"             => 1,
+			  "COLLATION"           => 1,
+			  "COLUMN"              => 1,
+			  "COLUMNS"             => 1,
+			  "COMMENT"             => 1,
+			  "COMMIT"              => 1,
+			  "COMPLETION"          => 1,
+			  "COMPRESS"            => 1,
+			  "CONNECT"             => 1,
+			  "CONNECTION"          => 1,
+			  "CONSTRAINT"          => 1,
+			  "CONSTRAINTS"         => 1,
+			  "CONSTRUCTOR"         => 1,
+			  "CONTINUE"            => 1,
+			  "CONVERT"             => 1,
+			  "CORRESPONDING"       => 1,
+			  "COUNT"               => 1,
+			  "CREATE"              => 1,
+			  "CROSS"               => 1,
+			  "CUBE"                => 1,
+			  "CURRENT"             => 1,
+			  "CURRENT_DATE"        => 1,
+			  "CURRENT_PATH"        => 1,
+			  "CURRENT_ROLE"        => 1,
+			  "CURRENT_TIME"        => 1,
+			  "CURRENT_TIMESTAMP"   => 1,
+			  "CURRENT_USER"        => 1,
+			  "CURSOR"              => 1,
+			  "CYCLE"               => 1,
+			  "DATA"                => 1,
+			  "DATABASE"            => 1,
+			  "DATABASES"           => 1,
+			  "DATE"                => 1,
+			  "DAY"                 => 1,
+			  "DAY_HOUR"            => 1,
+			  "DAY_MINUTE"          => 1,
+			  "DAY_SECOND"          => 1,
+			  "DEALLOCATE"          => 1,
+			  "DEC"                 => 1,
+			  "DECIMAL"             => 1,
+			  "DECLARE"             => 1,
+			  "DEFAULT"             => 1,
+			  "DEFERRABLE"          => 1,
+			  "DEFERRED"            => 1,
+			  "DELAYED"             => 1,
+			  "DELETE"              => 1,
+			  "DEPTH"               => 1,
+			  "DEREF"               => 1,
+			  "DESC"                => 1,
+			  "DESCRIBE"            => 1,
+			  "DESCRIPTOR"          => 1,
+			  "DESTROY"             => 1,
+			  "DESTRUCTOR"          => 1,
+			  "DETERMINISTIC"       => 1,
+			  "DIAGNOSTICS"         => 1,
+			  "DICTIONARY"          => 1,
+			  "DISCONNECT"          => 1,
+			  "DISTINCT"            => 1,
+			  "DISTINCTROW"         => 1,
+			  "DIV"                 => 1,
+			  "DO"                  => 1,
+			  "DOMAIN"              => 1,
+			  "DOUBLE"              => 1,
+			  "DROP"                => 1,
+			  "DUAL"                => 1,
+			  "DYNAMIC"             => 1,
+			  "EACH"                => 1,
+			  "ELSE"                => 1,
+			  "ENCLOSED"            => 1,
+			  "END"                 => 1,
+			  "END-EXEC"            => 1,
+			  "EQUALS"              => 1,
+			  "ERRORS"              => 1,
+			  "ESCAPE"              => 1,
+			  "ESCAPED"             => 1,
+			  "EVERY"               => 1,
+			  "EXCEPT"              => 1,
+			  "EXCEPTION"           => 1,
+			  "EXCLUSIVE"           => 1,
+			  "EXEC"                => 1,
+			  "EXECUTE"             => 1,
+			  "EXISTS"              => 1,
+			  "EXPLAIN"             => 1,
+			  "EXTERNAL"            => 1,
+			  "EXTRACT"             => 1,
+			  "FALSE"               => 1,
+			  "FETCH"               => 1,
+			  "FIELDS"              => 1,
+			  "FILE"                => 1,
+			  "FIRST"               => 1,
+			  "FLOAT"               => 1,
+			  "FLOAT4"              => 1,
+			  "FLOAT8"              => 1,
+			  "FOR"                 => 1,
+			  "FORCE"               => 1,
+			  "FOREIGN"             => 1,
+			  "FOUND"               => 1,
+			  "FREE"                => 1,
+			  "FROM"                => 1,
+			  "FULL"                => 1,
+			  "FULLTEXT"            => 1,
+			  "FUNCTION"            => 1,
+			  "GENERAL"             => 1,
+			  "GET"                 => 1,
+			  "GLOBAL"              => 1,
+			  "GO"                  => 1,
+			  "GOTO"                => 1,
+			  "GRANT"               => 1,
+			  "GROUP"               => 1,
+			  "GROUPING"            => 1,
+			  "HASH"                => 1,
+			  "HAVING"              => 1,
+			  "HIGH_PRIORITY"       => 1,
+			  "HOST"                => 1,
+			  "HOUR"                => 1,
+			  "HOUR_MINUTE"         => 1,
+			  "HOUR_SECOND"         => 1,
+			  "IDENTIFIED"          => 1,
+			  "IDENTITY"            => 1,
+			  "IF"                  => 1,
+			  "IGNORE"              => 1,
+			  "IMMEDIATE"           => 1,
+			  "IN"                  => 1,
+			  "INCREMENT"           => 1,
+			  "INDEX"               => 1,
+			  "INDICATOR"           => 1,
+			  "INFILE"              => 1,
+			  "INITIAL"             => 1,
+			  "INITIALIZE"          => 1,
+			  "INITIALLY"           => 1,
+			  "INNER"               => 1,
+			  "INNODB"              => 1,
+			  "INOUT"               => 1,
+			  "INPUT"               => 1,
+			  "INSENSITIVE"         => 1,
+			  "INSERT"              => 1,
+			  "INT"                 => 1,
+			  "INT1"                => 1,
+			  "INT2"                => 1,
+			  "INT3"                => 1,
+			  "INT4"                => 1,
+			  "INT8"                => 1,
+			  "INTEGER"             => 1,
+			  "INTERSECT"           => 1,
+			  "INTERVAL"            => 1,
+			  "INTO"                => 1,
+			  "IS"                  => 1,
+			  "ISOLATION"           => 1,
+			  "ITERATE"             => 1,
+			  "JOIN"                => 1,
+			  "KEY"                 => 1,
+			  "KEYS"                => 1,
+			  "KEY_NO"              => 1,
+			  "KILL"                => 1,
+			  "LANGUAGE"            => 1,
+			  "LARGE"               => 1,
+			  "LAST"                => 1,
+			  "LATERAL"             => 1,
+			  "LEADING"             => 1,
+			  "LEFT"                => 1,
+			  "LESS"                => 1,
+			  "LEVEL"               => 1,
+			  "LIKE"                => 1,
+			  "LIMIT"               => 1,
+			  "LINES"               => 1,
+			  "LOAD"                => 1,
+			  "LOCAL"               => 1,
+			  "LOCALTIME"           => 1,
+			  "LOCALTIMESTAMP"      => 1,
+			  "LOCATOR"             => 1,
+			  "LOCK"                => 1,
+			  "LONG"                => 1,
+			  "LONGBLOB"            => 1,
+			  "LONGTEXT"            => 1,
+			  "LOWER"               => 1,
+			  "LOW_PRIORITY"        => 1,
+			  "MAP"                 => 1,
+			  "MASTER_SERVER_ID"    => 1,
+			  "MATCH"               => 1,
+			  "MAX"                 => 1,
+			  "MAXEXTENTS"          => 1,
+			  "MEDIUMBLOB"          => 1,
+			  "MEDIUMINT"           => 1,
+			  "MEDIUMTEXT"          => 1,
+			  "MIDDLEINT"           => 1,
+			  "MIN"                 => 1,
+			  "MINUS"               => 1,
+			  "MINUTE"              => 1,
+			  "MINUTE_SECOND"       => 1,
+			  "MOD"                 => 1,
+			  "MODE"                => 1,
+			  "MODIFIES"            => 1,
+			  "MODIFY"              => 1,
+			  "MODULE"              => 1,
+			  "MONTH"               => 1,
+			  "MRG_MYISAM"          => 1,
+			  "NAMES"               => 1,
+			  "NATIONAL"            => 1,
+			  "NATURAL"             => 1,
+			  "NCHAR"               => 1,
+			  "NCLOB"               => 1,
+			  "NEW"                 => 1,
+			  "NEXT"                => 1,
+			  "NO"                  => 1,
+			  "NOAUDIT"             => 1,
+			  "NOCOMPRESS"          => 1,
+			  "NONE"                => 1,
+			  "NOT"                 => 1,
+			  "NOWAIT"              => 1,
+			  "NULL"                => 1,
+			  "NULLIF"              => 1,
+			  "NUMBER"              => 1,
+			  "NUMERIC"             => 1,
+			  "OBJECT"              => 1,
+			  "OCTET_LENGTH"        => 1,
+			  "OF"                  => 1,
+			  "OFF"                 => 1,
+			  "OFFLINE"             => 1,
+			  "OFFSET"              => 1,
+			  "OID"                 => 1,
+			  "OLD"                 => 1,
+			  "ON"                  => 1,
+			  "ONLINE"              => 1,
+			  "ONLY"                => 1,
+			  "OPEN"                => 1,
+			  "OPERATION"           => 1,
+			  "OPTIMIZE"            => 1,
+			  "OPTION"              => 1,
+			  "OPTIONALLY"          => 1,
+			  "OR"                  => 1,
+			  "ORDER"               => 1,
+			  "ORDINALITY"          => 1,
+			  "OUT"                 => 1,
+			  "OUTER"               => 1,
+			  "OUTFILE"             => 1,
+			  "OUTPUT"              => 1,
+			  "OVERLAPS"            => 1,
+			  "PAD"                 => 1,
+			  "PARAMETER"           => 1,
+			  "PARAMETERS"          => 1,
+			  "PARTIAL"             => 1,
+			  "PATH"                => 1,
+			  "PCTFREE"             => 1,
+			  "PLACING"             => 1,
+			  "POSITION"            => 1,
+			  "POSTFIX"             => 1,
+			  "PRECISION"           => 1,
+			  "PREFIX"              => 1,
+			  "PREORDER"            => 1,
+			  "PREPARE"             => 1,
+			  "PRESERVE"            => 1,
+			  "PRIMARY"             => 1,
+			  "PRIOR"               => 1,
+			  "PRIVILEGES"          => 1,
+			  "PROCEDURE"           => 1,
+			  "PUBLIC"              => 1,
+			  "PURGE"               => 1,
+			  "RAW"                 => 1,
+			  "READ"                => 1,
+			  "READS"               => 1,
+			  "REAL"                => 1,
+			  "RECURSIVE"           => 1,
+			  "REF"                 => 1,
+			  "REFERENCES"          => 1,
+			  "REFERENCING"         => 1,
+			  "REGEXP"              => 1,
+			  "RELATIVE"            => 1,
+			  "RENAME"              => 1,
+			  "REPLACE"             => 1,
+			  "REQUIRE"             => 1,
+			  "RESOURCE"            => 1,
+			  "RESTRICT"            => 1,
+			  "RESULT"              => 1,
+			  "RETURN"              => 1,
+			  "RETURNS"             => 1,
+			  "REVOKE"              => 1,
+			  "RIGHT"               => 1,
+			  "RLIKE"               => 1,
+			  "ROLE"                => 1,
+			  "ROLLBACK"            => 1,
+			  "ROLLUP"              => 1,
+			  "ROUTINE"             => 1,
+			  "ROW"                 => 1,
+			  "ROWID"               => 1,
+			  "ROWLABEL"            => 1,
+			  "ROWNUM"              => 1,
+			  "ROWS"                => 1,
+			  "RTREE"               => 1,
+			  "SAVEPOINT"           => 1,
+			  "SCHEMA"              => 1,
+			  "SCOPE"               => 1,
+			  "SCROLL"              => 1,
+			  "SEARCH"              => 1,
+			  "SECOND"              => 1,
+			  "SECTION"             => 1,
+			  "SELECT"              => 1,
+			  "SEQUENCE"            => 1,
+			  "SESSION"             => 1,
+			  "SESSION_USER"        => 1,
+			  "SET"                 => 1,
+			  "SETS"                => 1,
+			  "SHARE"               => 1,
+			  "SHOW"                => 1,
+			  "SIZE"                => 1,
+			  "SMALLINT"            => 1,
+			  "SOME"                => 1,
+			  "SONAME"              => 1,
+			  "SPACE"               => 1,
+			  "SPATIAL"             => 1,
+			  "SPECIFIC"            => 1,
+			  "SPECIFICTYPE"        => 1,
+			  "SQL"                 => 1,
+			  "SQLCODE"             => 1,
+			  "SQLERROR"            => 1,
+			  "SQLEXCEPTION"        => 1,
+			  "SQLSTATE"            => 1,
+			  "SQLWARNING"          => 1,
+			  "SQL_BIG_RESULT"      => 1,
+			  "SQL_CALC_FOUND_ROWS" => 1,
+			  "SQL_SMALL_RESULT"    => 1,
+			  "SSL"                 => 1,
+			  "START"               => 1,
+			  "STARTING"            => 1,
+			  "STATE"               => 1,
+			  "STATEMENT"           => 1,
+			  "STATIC"              => 1,
+			  "STRAIGHT_JOIN"       => 1,
+			  "STRIPED"             => 1,
+			  "STRUCTURE"           => 1,
+			  "SUBSTRING"           => 1,
+			  "SUCCESSFUL"          => 1,
+			  "SUM"                 => 1,
+			  "SYNONYM"             => 1,
+			  "SYSDATE"             => 1,
+			  "SYSTEM_USER"         => 1,
+			  "TABLE"               => 1,
+			  "TABLES"              => 1,
+			  "TEMPORARY"           => 1,
+			  "TERMINATE"           => 1,
+			  "TERMINATED"          => 1,
+			  "TEXT"                => 1,
+			  "THAN"                => 1,
+			  "THEN"                => 1,
+			  "TID"                 => 1,
+			  "TIME"                => 1,
+			  "TIMESTAMP"           => 1,
+			  "TIMEZONE_HOUR"       => 1,
+			  "TIMEZONE_MINUTE"     => 1,
+			  "TINYBLOB"            => 1,
+			  "TINYINT"             => 1,
+			  "TINYTEXT"            => 1,
+			  "TO"                  => 1,
+			  "TRAILING"            => 1,
+			  "TRANSACTION"         => 1,
+			  "TRANSLATE"           => 1,
+			  "TRANSLATION"         => 1,
+			  "TREAT"               => 1,
+			  "TRIGGER"             => 1,
+			  "TRIM"                => 1,
+			  "TRUE"                => 1,
+			  "TYPES"               => 1,
+			  "UID"                 => 1,
+			  "UNDER"               => 1,
+			  "UNION"               => 1,
+			  "UNIQUE"              => 1,
+			  "UNKNOWN"             => 1,
+			  "UNLOCK"              => 1,
+			  "UNNEST"              => 1,
+			  "UNSIGNED"            => 1,
+			  "UPDATE"              => 1,
+			  "UPPER"               => 1,
+			  "USAGE"               => 1,
+			  "USE"                 => 1,
+			  "USER"                => 1,
+			  "USER_RESOURCES"      => 1,
+			  "USING"               => 1,
+			  "VALIDATE"            => 1,
+			  "VALUE"               => 1,
+			  "VALUES"              => 1,
+			  "VARBINARY"           => 1,
+			  "VARCHAR"             => 1,
+			  "VARCHAR2"            => 1,
+			  "VARCHARACTER"        => 1,
+			  "VARIABLE"            => 1,
+			  "VARYING"             => 1,
+			  "VIEW"                => 1,
+			  "WARNINGS"            => 1,
+			  "WHEN"                => 1,
+			  "WHENEVER"            => 1,
+			  "WHERE"               => 1,
+			  "WITH"                => 1,
+			  "WITHOUT"             => 1,
+			  "WORK"                => 1,
+			  "WRITE"               => 1,
+			  "XID"                 => 1,
+			  "XOR"                 => 1,
+			  "YEAR"                => 1,
+			  "YEAR_MONTH"          => 1,
+			  "ZEROFILL"            => 1,
+			  "ZONE"                => 1,
+			  );
+      if ($is_keyword[strtoupper($identifier)]) {
+	return true;
+      }
+      return false;
+    }
+
+/**/
+
+    /**
+     * ܥ̾Ĵ٤
+     *
+     *@access public
+     *@param string $identifier Ĵ٤̾
+     */
+    function valid_sql_identifier($identifier) {
+      if($this->is_sql_keyword($identifier)) {
+	return 1;
+      }
+      if(strlen($identifier) > 31) {
+	return 2;
+      }
+      return 0;
+    }
+
+
     function _debug ($fnname,&$val) {
       if ($this->debug) {
 	echo "<B>$fnname</B>:<BR><TT>".
@@ -1212,6 +1770,8 @@
       }
     }
   }
+
+  /* obsolete: this class is for backward-compatiblity */
   class SQL extends Dino_SQL{
     function SQL(){
       parent::Dino_SQL();
